@@ -159,7 +159,7 @@ fetch_items_summary_cb (FetchItemsCallbackData *item_data, gpointer data)
 	MailItem *item = g_new0(MailItem , 1);
 
 	if (camel_debug_start("mapi:folder")) {
-		exchange_mapi_debug_property_dump (item_data->properties);
+		exchange_mapi_debug_dump_properties (item_data->conn, item_data->fid, item_data->properties);
 		camel_debug_end();
 	}
 
@@ -317,7 +317,6 @@ mapi_update_cache (CamelFolder *folder, GSList *list, CamelFolderChangeInfo **ch
 	CamelMapiStore *mapi_store;
 	CamelStore *parent_store;
 
-	guint32 status_flags = 0;
 	CamelFolderChangeInfo *changes = NULL;
 	gboolean exists = FALSE;
 	GString *str = g_string_new (NULL);
@@ -343,19 +342,16 @@ mapi_update_cache (CamelFolder *folder, GSList *list, CamelFolderChangeInfo **ch
 	camel_operation_push_message (
 		cancellable,
 		_("Updating local summary cache for new messages in %s"),
-		camel_folder_get_name (folder));
+		camel_folder_get_display_name (folder));
 
 	for (; item_list != NULL; item_list = g_slist_next (item_list) ) {
 		MailItem *temp_item;
 		MailItem *item;
 		gchar *msg_uid;
-		guint64 id;
 
 		exists = FALSE;
-		status_flags = 0;
 
 		temp_item = (MailItem *)item_list->data;
-		id = temp_item->mid;
 		item = temp_item;
 
 		camel_operation_progress (cancellable, (100*i)/total_items);
@@ -510,9 +506,9 @@ mapi_utils_do_flags_diff (flags_diff_t *diff, guint32 old, guint32 _new)
 	diff->bits = _new & diff->changed;
 }
 
-struct mapi_update_deleted_msg {
-	CamelSessionThreadMsg msg;
+typedef struct _SyncDeletedData SyncDeletedData;
 
+struct _SyncDeletedData {
 	CamelFolder *folder;
 	mapi_id_t folder_id;
 	gboolean need_refresh;
@@ -545,37 +541,44 @@ deleted_items_sync_cb (FetchItemsCallbackData *item_data, gpointer data)
 }
 
 static void
-mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
+mapi_sync_deleted (CamelSession *session,
+                   GCancellable *cancellable,
+                   SyncDeletedData *data,
+                   GError **error)
 {
-	struct mapi_update_deleted_msg *m = (struct mapi_update_deleted_msg *)msg;
-
 	CamelMapiStore *mapi_store;
 	CamelMapiFolder *mapi_folder;
 	CamelFolderChangeInfo *changes = NULL;
 	CamelMessageInfo *info = NULL;
 	CamelStore *parent_store;
+	CamelServiceConnectionStatus status;
+	CamelService *service;
 
 	guint32 index, count, options = 0;
 	GHashTable *server_messages = NULL;
 	const gchar *uid = NULL;
 	gboolean flags_changed = FALSE;
 
-	parent_store = camel_folder_get_parent_store (m->folder);
+	parent_store = camel_folder_get_parent_store (data->folder);
 
-	mapi_folder = CAMEL_MAPI_FOLDER (m->folder);
+	mapi_folder = CAMEL_MAPI_FOLDER (data->folder);
 	mapi_store = CAMEL_MAPI_STORE (parent_store);
 
-	if (!camel_offline_store_get_online (CAMEL_OFFLINE_STORE (mapi_store)) ||
-			((CamelService *)mapi_store)->status == CAMEL_SERVICE_DISCONNECTED) {
+	service = CAMEL_SERVICE (mapi_store);
+	status = camel_service_get_connection_status (service);
 
-		return;
+	if (!camel_offline_store_get_online (CAMEL_OFFLINE_STORE (mapi_store)) ||
+			status == CAMEL_SERVICE_DISCONNECTED) {
+
+		goto exit;
 	}
 
 	camel_operation_push_message (
-		NULL, _("Retrieving message IDs from server for %s"),
-		camel_folder_get_name (m->folder));
+		cancellable,
+		_("Retrieving message IDs from server for %s"),
+		camel_folder_get_display_name (data->folder));
 
-	camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	if (mapi_folder->type & CAMEL_MAPI_FOLDER_PUBLIC)
 		options |= MAPI_OPTIONS_USE_PFSTORE;
@@ -583,34 +586,35 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 	server_messages = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
 	/*Get the UID list from server.*/
-	exchange_mapi_connection_fetch_items (camel_mapi_store_get_exchange_connection (mapi_store), m->folder_id, NULL, NULL,
+	exchange_mapi_connection_fetch_items (camel_mapi_store_get_exchange_connection (mapi_store), data->folder_id, NULL, NULL,
 					       NULL, NULL,
 					       deleted_items_sync_cb, server_messages,
 					       options | MAPI_OPTIONS_DONT_OPEN_MESSAGE, NULL);
 
-	camel_operation_pop_message (NULL);
+	camel_operation_pop_message (cancellable);
 
-	camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	/* Check if we have to stop */
-	if (camel_operation_cancel_check (NULL) || camel_application_is_exiting) {
+	if (g_cancellable_is_cancelled (cancellable) || camel_application_is_exiting) {
 		g_hash_table_destroy (server_messages);
-		return;
+		goto exit;
 	}
 
 	changes = camel_folder_change_info_new ();
 
-	count = camel_folder_summary_count (m->folder->summary);
+	count = camel_folder_summary_count (data->folder->summary);
 	camel_operation_push_message (
-		NULL, _("Removing deleted messages from cache in %s"),
-		camel_folder_get_name (m->folder));
+		cancellable,
+		_("Removing deleted messages from cache in %s"),
+		camel_folder_get_display_name (data->folder));
 
 	/* Iterate over cache and check if the UID is in server*/
 	for (index = 0; index < count; index++) {
 		guint32 msg_flags;
 
 		/* Iterate in a reverse order, thus removal will not hurt */
-		info = camel_folder_summary_index (m->folder->summary, count - index - 1);
+		info = camel_folder_summary_index (data->folder->summary, count - index - 1);
 		if (!info) continue; /*This is bad. *Should* not happen*/
 
 		uid = camel_message_info_uid (info);
@@ -623,11 +627,11 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 
 		/* If it is not in server list, clean our cache */
 		if (!msg_flags) {
-			camel_folder_summary_lock (m->folder->summary, CAMEL_FOLDER_SUMMARY_SUMMARY_LOCK);
-			camel_folder_summary_remove_uid (m->folder->summary, uid);
+			camel_folder_summary_lock (data->folder->summary, CAMEL_FOLDER_SUMMARY_SUMMARY_LOCK);
+			camel_folder_summary_remove_uid (data->folder->summary, uid);
 			camel_data_cache_remove (mapi_folder->cache, "cache", uid, NULL);
 			camel_folder_change_info_remove_uid (changes, uid);
-			camel_folder_summary_unlock (m->folder->summary, CAMEL_FOLDER_SUMMARY_SUMMARY_LOCK);
+			camel_folder_summary_unlock (data->folder->summary, CAMEL_FOLDER_SUMMARY_SUMMARY_LOCK);
 		} else {
 			CamelMapiMessageInfo *mapi_info = (CamelMapiMessageInfo *) info;
 
@@ -643,65 +647,51 @@ mapi_sync_deleted (CamelSession *session, CamelSessionThreadMsg *msg)
 		camel_message_info_free (info);
 
 		/* Progress update */
-		camel_operation_progress (NULL, (index * 100)/count); /* ;-) */
+		camel_operation_progress (cancellable, (index * 100)/count); /* ;-) */
 
 		/* Check if we have to stop */
-		if (camel_operation_cancel_check (NULL) || camel_application_is_exiting) {
+		if (g_cancellable_is_cancelled (cancellable) || camel_application_is_exiting) {
 			g_hash_table_destroy (server_messages);
 			if (camel_folder_change_info_changed (changes))
-				camel_folder_changed (m->folder, changes);
+				camel_folder_changed (data->folder, changes);
 			camel_folder_change_info_free (changes);
-			return;
+			goto exit;
 		}
 	}
 
-	camel_operation_pop_message (NULL);
+	camel_operation_pop_message (cancellable);
 
 	if (camel_folder_change_info_changed (changes)) {
 		if (flags_changed)
 			camel_mapi_summary_update_store_info_counts (CAMEL_MAPI_SUMMARY (CAMEL_FOLDER (mapi_folder)->summary));
-		camel_folder_changed (m->folder, changes);
+		camel_folder_changed (data->folder, changes);
 	}
 	camel_folder_change_info_free (changes);
 
-	m->need_refresh = camel_folder_summary_count (m->folder->summary) != g_hash_table_size (server_messages);
+	data->need_refresh = camel_folder_summary_count (data->folder->summary) != g_hash_table_size (server_messages);
 
 	g_hash_table_destroy (server_messages);
-}
 
-static void
-mapi_sync_deleted_free (CamelSession *session, CamelSessionThreadMsg *msg)
-{
-	struct mapi_update_deleted_msg *m = (struct mapi_update_deleted_msg *)msg;
-	CamelStore *parent_store;
-
-	parent_store = camel_folder_get_parent_store (m->folder);
-
-	if (m->need_refresh) {
-		CamelMapiSummary *mapi_summary = CAMEL_MAPI_SUMMARY (m->folder->summary);
+exit:
+	if (data->need_refresh) {
+		CamelMapiSummary *mapi_summary = CAMEL_MAPI_SUMMARY (data->folder->summary);
 		if (mapi_summary) {
-			GError *local_error = NULL;
-
 			camel_service_lock (CAMEL_SERVICE (parent_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 			g_free (mapi_summary->sync_time_stamp);
 			mapi_summary->sync_time_stamp = NULL;
 			camel_service_unlock (CAMEL_SERVICE (parent_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
-			/* FIXME Need to pass a GCancellable here. */
-			if (!mapi_refresh_folder (m->folder, NULL, &local_error)) {
-				g_warning ("%s: %s", G_STRFUNC, local_error->message);
-				g_error_free (local_error);
-			}
+			mapi_refresh_folder (data->folder, cancellable, error);
 		}
 	}
-
-	g_object_unref (m->folder);
 }
 
-static CamelSessionThreadOps deleted_items_sync_ops = {
-	mapi_sync_deleted,
-	mapi_sync_deleted_free,
-};
+static void
+mapi_sync_deleted_data_free (SyncDeletedData *data)
+{
+	g_object_unref (data->folder);
+	g_slice_free (SyncDeletedData, data);
+}
 
 static gboolean
 mapi_camel_get_summary_list (ExchangeMapiConnection *conn, mapi_id_t fid, TALLOC_CTX *mem_ctx, struct SPropTagArray *props, gpointer data)
@@ -739,7 +729,7 @@ camel_mapi_folder_fetch_summary (CamelStore *store, CamelFolder *folder, const m
 
 	/*TODO : Check for online state*/
 
-	camel_operation_push_message (cancellable, _("Fetching summary information for new messages in %s"), camel_folder_get_name (folder));
+	camel_operation_push_message (cancellable, _("Fetching summary information for new messages in %s"), camel_folder_get_display_name (folder));
 
 	camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
@@ -762,6 +752,7 @@ mapi_refresh_folder(CamelFolder *folder, GCancellable *cancellable, GError **err
 	CamelMapiStore *mapi_store;
 	CamelMapiFolder *mapi_folder;
 	CamelMapiSummary *mapi_summary;
+	CamelService *service;
 	CamelSession *session;
 	CamelStore *parent_store;
 
@@ -773,7 +764,6 @@ mapi_refresh_folder(CamelFolder *folder, GCancellable *cancellable, GError **err
 	TALLOC_CTX *mem_ctx = NULL;
 	struct mapi_SRestriction *res = NULL;
 	struct SSortOrderSet *sort = NULL;
-	struct mapi_update_deleted_msg *deleted_items_op_msg;
 	fetch_items_data *fetch_data = g_new0 (fetch_items_data, 1);
 	const gchar *folder_id = NULL;
 	const gchar *full_name;
@@ -786,7 +776,9 @@ mapi_refresh_folder(CamelFolder *folder, GCancellable *cancellable, GError **err
 	mapi_summary = CAMEL_MAPI_SUMMARY (folder->summary);
 
 	is_proxy = parent_store->flags & CAMEL_STORE_PROXY;
-	session = CAMEL_SERVICE (parent_store)->session;
+
+	service = CAMEL_SERVICE (parent_store);
+	session = camel_service_get_session (service);
 
 	if (!camel_offline_store_get_online (CAMEL_OFFLINE_STORE (mapi_store)))
 		goto end1;
@@ -806,7 +798,7 @@ mapi_refresh_folder(CamelFolder *folder, GCancellable *cancellable, GError **err
 		mapi_folder->need_refresh = TRUE;
 	}
 
-	camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 	is_locked = TRUE;
 
 	if (!camel_mapi_store_connected (mapi_store, NULL))
@@ -814,6 +806,7 @@ mapi_refresh_folder(CamelFolder *folder, GCancellable *cancellable, GError **err
 
 	/*Get the New Items*/
 	if (!is_proxy) {
+		SyncDeletedData *sync_data;
 		mapi_id_t temp_folder_id;
 		guint32 options = 0;
 		GError *mapi_error = NULL;
@@ -836,11 +829,9 @@ mapi_refresh_folder(CamelFolder *folder, GCancellable *cancellable, GError **err
 
 			//Creation time ?
 			set_SPropValue_proptag_date_timeval (&sprop, PR_LAST_MODIFICATION_TIME, &t);
-			cast_mapi_SPropValue (
-				#ifdef HAVE_MEMCTX_ON_CAST_MAPI_SPROPVALUE
-				mem_ctx,
-				#endif
-				&(res->res.resProperty.lpProp), &sprop);
+			cast_mapi_SPropValue (mem_ctx,
+					      &(res->res.resProperty.lpProp),
+					      &sprop);
 
 		}
 
@@ -897,18 +888,20 @@ mapi_refresh_folder(CamelFolder *folder, GCancellable *cancellable, GError **err
 		camel_folder_summary_touch (folder->summary);
 		update_store_summary (folder, NULL);
 
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		is_locked = FALSE;
 
 		/* Downsync deleted items */
-		deleted_items_op_msg = camel_session_thread_msg_new (session, &deleted_items_sync_ops,
-							 sizeof (*deleted_items_op_msg));
-		deleted_items_op_msg->folder = folder;
-		deleted_items_op_msg->folder_id = temp_folder_id;
-		deleted_items_op_msg->need_refresh = FALSE;
-		g_object_ref (folder);
+		sync_data = g_slice_new0 (SyncDeletedData);
+		sync_data->folder = g_object_ref (folder);
+		sync_data->folder_id = temp_folder_id;
+		sync_data->need_refresh = FALSE;
 
-		camel_session_thread_queue (session, &deleted_items_op_msg->msg, 0);
+		camel_session_submit_job (
+			session,
+			(CamelSessionCallback) mapi_sync_deleted,
+			sync_data,
+			(GDestroyNotify) mapi_sync_deleted_data_free);
 
 		camel_folder_changed (folder, fetch_data->changes);
 		camel_folder_change_info_free (fetch_data->changes);
@@ -916,7 +909,7 @@ mapi_refresh_folder(CamelFolder *folder, GCancellable *cancellable, GError **err
 
 end1:
 	if (is_locked)
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	g_slist_foreach (fetch_data->items_list, (GFunc) mail_item_free, NULL);
 	g_slist_free (fetch_data->items_list);
@@ -1044,7 +1037,7 @@ mapi_folder_constructed (GObject *object)
 	folder = CAMEL_FOLDER (object);
 	full_name = camel_folder_get_full_name (folder);
 	parent_store = camel_folder_get_parent_store (folder);
-	url = CAMEL_SERVICE (parent_store)->url;
+	url = camel_service_get_camel_url (CAMEL_SERVICE (parent_store));
 
 	description = g_strdup_printf (
 		"%s@%s:%s", url->user, url->host, full_name);
@@ -1343,9 +1336,10 @@ mapi_folder_get_message_sync (CamelFolder *folder,
 		GError *local_error = NULL;
 
 		msg = camel_mime_message_new ();
-		camel_stream_reset (stream, NULL);
+
+		g_seekable_seek (G_SEEKABLE (stream), 0, G_SEEK_SET, NULL, NULL);
 		camel_stream_write_to_stream (cache_stream, stream, cancellable, NULL);
-		camel_stream_reset (stream, NULL);
+		g_seekable_seek (G_SEEKABLE (stream), 0, G_SEEK_SET, NULL, NULL);
 		if (!camel_data_wrapper_construct_from_stream_sync ((CamelDataWrapper *) msg, stream, cancellable, &local_error)) {
 			if (g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
 				g_object_unref (msg);
@@ -1448,7 +1442,7 @@ mapi_folder_get_message_sync (CamelFolder *folder,
 			   composing message in memory, but when they are read from the cache
 			   they appear properly in the UI. */
 			msg2 = camel_mime_message_new ();
-			camel_stream_reset (cache_stream, NULL);
+			g_seekable_seek (G_SEEKABLE (cache_stream), 0, G_SEEK_SET, NULL, NULL);
 			if (!camel_data_wrapper_construct_from_stream_sync (CAMEL_DATA_WRAPPER (msg2), cache_stream, cancellable, NULL)) {
 				g_object_unref (msg2);
 			} else {
@@ -1489,6 +1483,8 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 	CamelMapiMessageInfo *mapi_info = NULL;
 	CamelStore *parent_store;
 	CamelFolderChangeInfo *changes = NULL;
+	CamelServiceConnectionStatus status;
+	CamelService *service;
 
 	GSList *read_items = NULL, *unread_items = NULL, *to_free = NULL, *junk_items = NULL, *deleted_items = NULL, *l;
 	flags_diff_t diff, unset_flags;
@@ -1506,8 +1502,11 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 	mapi_folder = CAMEL_MAPI_FOLDER (folder);
 	mapi_store = CAMEL_MAPI_STORE (parent_store);
 
+	service = CAMEL_SERVICE (mapi_store);
+	status = camel_service_get_connection_status (service);
+
 	if (!camel_offline_store_get_online (CAMEL_OFFLINE_STORE (mapi_store)) ||
-			((CamelService *)mapi_store)->status == CAMEL_SERVICE_DISCONNECTED) {
+			status == CAMEL_SERVICE_DISCONNECTED) {
 		return update_store_summary (folder, error);
 	}
 
@@ -1517,12 +1516,12 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 	folder_id =  camel_mapi_store_folder_id_lookup (mapi_store, full_name);
 	exchange_mapi_util_mapi_id_from_string (folder_id, &fid);
 
-	camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 	if (!camel_mapi_store_connected (mapi_store, NULL)) {
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		return TRUE;
 	}
-	camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	is_junk_folder = (mapi_folder->type & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_JUNK;
 
@@ -1597,20 +1596,20 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 	*/
 
 	if (read_items) {
-		camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		exchange_mapi_connection_set_flags (camel_mapi_store_get_exchange_connection (mapi_store), 0, fid, options, read_items, 0, NULL);
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 	}
 
 	if (unread_items) {
-		camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		exchange_mapi_connection_set_flags (camel_mapi_store_get_exchange_connection (mapi_store), 0, fid, options, unread_items, CLEAR_READ_FLAG, NULL);
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 	}
 
 	/* Remove messages from server*/
 	if (deleted_items) {
-		camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		if ((mapi_folder->type & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_TRASH) {
 			exchange_mapi_connection_remove_items (camel_mapi_store_get_exchange_connection (mapi_store), 0, fid, options, deleted_items, NULL);
 		} else {
@@ -1625,17 +1624,17 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 			}
 		}
 
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 	}
 
 	if (junk_items) {
 		mapi_id_t junk_fid = 0;
 		GError *err = NULL;
 
-		camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 		exchange_mapi_util_mapi_id_from_string (camel_mapi_store_system_folder_fid (mapi_store, olFolderJunk), &junk_fid);
 		exchange_mapi_connection_move_items (camel_mapi_store_get_exchange_connection (mapi_store), fid, options, junk_fid, 0, junk_items, &err);
-		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 		/* in junk_items are only emails which are not deleted */
 		deleted_items = g_slist_concat (deleted_items, g_slist_copy (junk_items));
@@ -1677,9 +1676,9 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 		/* TODO */
 	}
 
-	camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 	success = update_store_summary (folder, error);
-	camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	return success;
 }
@@ -1793,6 +1792,10 @@ mapi_folder_transfer_messages_to_sync (CamelFolder *source,
 	g_slist_foreach (src_msg_ids, (GFunc) g_free, NULL);
 	g_slist_free (src_msg_ids);
 
+	/* update the destination folder to notice new messages */
+	if (success)
+		success = mapi_folder_refresh_info_sync (destination, cancellable, error);
+
 	return success;
 }
 
@@ -1850,10 +1853,18 @@ camel_mapi_folder_new (CamelStore *store, const gchar *folder_name, const gchar
 	CamelFolder	*folder = NULL;
 	CamelMapiFolder *mapi_folder;
 	CamelMapiStore  *mapi_store = (CamelMapiStore *) store;
+	CamelService    *service;
+	CamelSettings   *settings;
 
 	gchar *summary_file, *state_file;
 	const gchar *short_name;
 	CamelStoreInfo *si;
+	gboolean filter_inbox;
+
+	service = CAMEL_SERVICE (store);
+	settings = camel_service_get_settings (service);
+
+	filter_inbox = camel_store_settings_get_filter_inbox (CAMEL_STORE_SETTINGS (settings));
 
 	short_name = strrchr (folder_name, '/');
 	if (short_name)
@@ -1863,7 +1874,7 @@ camel_mapi_folder_new (CamelStore *store, const gchar *folder_name, const gchar
 
 	folder = g_object_new (
 		CAMEL_TYPE_MAPI_FOLDER,
-		"name", short_name, "full-name", folder_name,
+		"display-name", short_name, "full-name", folder_name,
 		"parent-store", store, NULL);
 
 	mapi_folder = CAMEL_MAPI_FOLDER(folder);
@@ -1904,7 +1915,7 @@ camel_mapi_folder_new (CamelStore *store, const gchar *folder_name, const gchar
 /*		return NULL; */
 /*	} */
 
-	if ((store->flags & CAMEL_STORE_FILTER_INBOX) != 0) {
+	if (filter_inbox) {
 		CamelFolderInfo *fi;
 
 		fi = camel_store_get_folder_info_sync (store, folder_name, 0, NULL, NULL);
